<?php
/**
 * @file
 * Form, validations and submission functions.
 */
/**
 * The instance add/edit form.
 * FORM
 *
 * @param $contextual_element
 *   If set then we are editing that element.  If NULL we are creating a new element.
 */
function contextual_elements_form_element($form, &$form_state, $contextual_element = NULL) {
  // Set an appropriate page title.
  $title = (isset($contextual_element->cid) ? contextual_elements_title('edit', $contextual_element) : t('Add an Element'));
  drupal_set_title($title);

  // If the $contextual_element object was not passed in (new element) and the session is set
  // with an object, we likely encountered an error and should repopulate the
  // fields for the user.
  if (!isset($contextual_element) && isset($_SESSION['contextual_elements_add']) && is_object($_SESSION['contextual_elements_add'])) {
    $contextual_element = $_SESSION['contextual_elements_add'];
    $contextual_element->roles = unserialize($contextual_element->roles);
    unset($_SESSION['contextual_elements_add']);
  }

  // If we are editing a element, set a redirect back to the view
  if (isset($contextual_element->cid) && is_numeric($contextual_element->cid)) {
    $form['#redirect'] = "admin/config/search/contextual-elements/{$contextual_element->cid}/view";
  }

  $form['cid'] = array(
    '#type' => 'value',
    '#value' => (isset($contextual_element->cid) ? $contextual_element->cid : NULL),
  );
  $form['s'] = array(
    '#type' => 'fieldset',
    '#title' => t('Status'),
    '#collapsible' => TRUE,
  );
  $form['s']['status'] = array(
    '#type' => 'select',
    '#title' => t('Status'),
    '#description' => '<strong>' . t('Disabled:')  . '</strong> ' . t('Elements are not rendered, regardless of filtering.') . '</br />' .
                      '<strong>' . t('Test Mode:') . '</strong> ' . t('HTML placeholders are rendered instead of the actual elements.') . '<br />' .
                      '<strong>' . t('Active:')    . '</strong> ' . t('Actual elements are rendered on the pages allowed by the filters.'),
    '#options' => _contextual_elements_opts('status'),
    '#default_value' => (isset($contextual_element->status) ? $contextual_element->status : 0),
  );
  $form['s']['debug'] = array(
    '#type' => 'select',
    '#title' => t('Debug'),
    '#description' => t('Display messages to users with debug permissions showing each filter result for this element.'),
    '#options' => _contextual_elements_opts('debug'),
    '#default_value' => (isset($contextual_element->debug) ? $contextual_element->debug : 0),
  );

  $form['info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Element Information'),
    '#collapsible' => TRUE,
  );
  $form['info']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Provide an administrative label for this set of elements.'),
    '#maxlength' => 255,
    '#default_value' =>  (isset($contextual_element->name) ? $contextual_element->name : ''),
    '#required' => TRUE,
    '#weight' => 0,
  );
  $form['info']['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#description' => t('Provide a description for this set of elements.'),
    '#maxlength' => 255,
    '#default_value' =>  (isset($contextual_element->description) ? $contextual_element->description : ''),
    '#required' => FALSE,
    '#weight' => 2,
  );
  $form['info']['header'] = array(
    '#type' => 'textarea',
    '#title' => t('Header Elements'),
    '#description' => t('Enter HTML elements to be added to the page header.  One item per line.'),
    '#default_value' => (isset($contextual_element->header) ? $contextual_element->header : ''),
    '#weight' => 4,
  );
  $form['info']['footer'] = array(
    '#type' => 'textarea',
    '#title' => t('Footer Elements'),
    '#description' => t('Enter HTML elements to be added to the page footer (closure).  One item per line.'),
    '#default_value' => (isset($contextual_element->footer) ? $contextual_element->footer : ''),
    '#weight' => 6,
  );
  $form['info']['weight'] = array(
    '#type' => 'select',
    '#title' => t('Weight'),
    '#options' => _contextual_elements_opts('weight'),
    '#default_value' => (isset($contextual_element->weight) ? $contextual_element->weight : 0),
    '#weight' => 8,
  );

  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filters'),
    '#collapsible' => TRUE,
  );
  $form['filters']['domains'] = array(
    '#type' => 'fieldset',
    '#title' => t('Domains'),
    '#collapsible' => TRUE,
  );
  $form['filters']['domains']['domain_inc'] = array(
    '#type' => 'radios',
    '#title' => t('Include or Exclude Domains'),
    '#description' => t('Choose whether to include the following domains in the allowed list or exclude them.'),
    '#options' => _contextual_elements_opts('domain_inc'),
    '#default_value' => (isset($contextual_element->domain_inc) ? $contextual_element->domain_inc : 0),
  );
  $form['filters']['domains']['domains'] = array(
    '#type' => 'textarea',
    '#title' => t('Domains'),
    '#description' => t('List domains names or host names one per line.'),
    '#rows' => 3,
    '#default_value' => (isset($contextual_element->domains) ? $contextual_element->domains : ''),
  );
  $form['filters']['p'] = array(
    '#type' => 'fieldset',
    '#title' => t('Paths'),
    '#collapsible' => TRUE,
  );
  $form['filters']['p']['page_inc'] = array(
    '#type' => 'radios',
    '#title' => t('Include or Exclude Paths'),
    '#description' => t('Choose whether to include the following paths in the allowed list or exclude them.'),
    '#options' => _contextual_elements_opts('page_inc'),
    '#default_value' => (isset($contextual_element->page_inc) ? $contextual_element->page_inc : 0),
  );
  $form['filters']['p']['pages'] = array(
    '#type' => 'textarea',
    '#title' => t('Paths'),
    '#description' => t('List paths one per line.'),
    '#rows' => 3,
    '#default_value' => (isset($contextual_element->pages) ? $contextual_element->pages : ''),
  );
  $form['filters']['r'] = array(
    '#type' => 'fieldset',
    '#title' => t('Roles'),
    '#collapsible' => TRUE,
  );
  $form['filters']['r']['role_inc'] = array(
    '#type' => 'radios',
    '#title' => t('Include or Exclude Roles'),
    '#description' => t('Choose whether to include or exclude the following roles.'),
    '#options' => _contextual_elements_opts('role_inc'),
    '#default_value' => (isset($contextual_element->role_inc) ? $contextual_element->role_inc : 0),
  );
  $form['filters']['r']['roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Roles'),
    '#description' => t('Select roles to include or exclude.'),
    '#options' => user_roles(),
    '#default_value' => (isset($contextual_element->roles) ? $contextual_element->roles : array()),
  );
  $form['filters']['users'] = array(
    '#type' => 'radios',
    '#title' => t('User Specific Filters'),
    '#description' => t('Users with the proper permissions may choose to opt-in or opt-out.'),
    '#options' => _contextual_elements_opts('users'),
    '#default_value' => (isset($contextual_element->users) ? $contextual_element->users : 0),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save elements'),
  );
  $form['cancel'] = array(
    '#type' => 'markup',
    '#value' => l('Cancel', 'admin/config/search/contextual-elements'),
  );

  return $form;
}

/**
 * The instance add/edit form.
 * VALIDATE
 *
 * @TODO: Create validation procedures?
 *  This module should be available only to admins so how much do we want to
 *  trust them or limit what they can place into the headers and footers?
 */
function contextual_elements_form_element_validate($form, &$form_state) {
  $vals = $form_state['values'];

  // Names must be unique
  $select = db_select('contextual_elements', 'c')->fields('c')->condition('name', $vals['name'])->condition('cid', $vals['cid'], '<>');
  $cnt = $select->countQuery()->execute()->fetchField();

  if ($cnt > 0) {
    form_set_error('name', t('Name %name is already taken.', array('%name' => $vals['name'])));
  }
}

/**
 * The instance add/edit form.
 * SUBMIT
 */
function contextual_elements_form_element_submit($form, &$form_state) {
  $vals = $form_state['values'];

  // We could just convert the $vals type to object and send that to the db,
  // but it is a safer practice to control exactly what data is being passed to
  // the db.
  $contextual_element               = new stdClass();
  $contextual_element->cid          = $vals['cid'];
  $contextual_element->status       = $vals['status'];
  $contextual_element->debug        = $vals['debug'];
  $contextual_element->name         = $vals['name'];
  $contextual_element->description  = $vals['description'];
  $contextual_element->header       = $vals['header'];
  $contextual_element->footer       = $vals['footer'];
  $contextual_element->weight       = $vals['weight'];
  $contextual_element->domain_inc   = $vals['domain_inc'];
  $contextual_element->domains      = $vals['domains'];
  $contextual_element->page_inc     = $vals['page_inc'];
  $contextual_element->pages        = $vals['pages'];
  $contextual_element->role_inc     = $vals['role_inc'];
  $contextual_element->roles        = serialize($vals['roles']);
  $contextual_element->users        = $vals['users'];

  contextual_elements_save($contextual_element);

  // Display different messages and redirects if this is an update vs an insert
  $args = array('%name' => $contextual_element->name);
  if (!is_numeric($vals['cid'])) {

    // New element
    if (is_numeric($contextual_element->cid)) {
      drupal_set_message(t('%name successfully created!', $args), 'status');
      drupal_goto('admin/config/search/contextual-elements');
    }
    else {
      // Store the data into the session so the "add" page can repopulate the
      // fields.
      $_SESSION['contextual_elements_add'] = $contextual_element;

      drupal_set_message(t('Error creating element %name.', $args), 'error');
      watchdog('contextual_elements', 'Error creating element %name.  CID was not assigned to new object.', $args, WATCHDOG_ERROR);
    }
  }
  else {
    drupal_set_message(t('%name was successfully updated.', $args));
  }
}

/**
 * Confirm form for deleting a contextual_element.
 * FORM
 */
function contextual_elements_delete_confirm_form($form, &$form_state, $contextual_element) {
  $form['contextual_element'] = array(
    '#type' => 'value',
    '#value' => $contextual_element,
  );
  $question = filter_xss(contextual_elements_title('delete', $contextual_element)) . '?';

  return confirm_form($form, $question, 'admin/config/search/contextual-elements');
}

/**
 * Confirm form for deleting a contextual_element.
 * SUBMIT
 */
function contextual_elements_delete_confirm_form_submit($form, &$form_state) {
  $vals = $form_state['values'];
  $args = array('%name' => $contextual_element->name);

  if ($vals['confirm'] == 1) {
    if ($vals['contextual_element']->cid) {
      contextual_elements_delete($vals['contextual_element']->cid);
    }
    else {
      drupal_set_message(t('Error deleting %name.', $args), 'error');
      watchdog('contextual_elements', 'Error deleting %name.  Missing CID.', $args, WATCHDOG_ERROR);
    }
  }
  drupal_goto('admin/config/search/contextual-elements');
}

/**
 * DnD table form for changing the order of the elements.
 * FORM
 */
function contextual_elements_instances_table_form($form_state) {
  $inst = contextual_elements_get_elements();

  foreach ($inst as $i) {
    $form['#tree'] = TRUE;
    $form['data'][$i->cid]['weight']= array(
      '#type' => 'select',
      '#options' => _contextual_elements_opts('weight'),
      '#default_value' => $i->weight,
      '#attributes' => array('class' => array('weight-drag')),
    );
    $form['data'][$i->cid]['status'] = array(
      '#type' => 'select',
      '#options' => _contextual_elements_opts('status'),
      '#default_value' => $i->status,
    );
    $form['data'][$i->cid]['debug'] = array(
      '#type' => 'select',
      '#options' => _contextual_elements_opts('debug'),
      '#default_value' => $i->debug,
    );
  }

  $form['headers'] = array(
    '#type' => 'value',
    '#value' => array(t('Element Name'), t('Status'), t('Debug'), t('Weight'), t('Operations')),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Changes'),
  );

  return $form;
}

/**
 * DnD table form for changing the order of the elements.
 * SUBMIT
 */
function contextual_elements_instances_table_form_submit($form, &$form_state) {
  $vals = $form_state['values'];

  foreach ($vals['data'] as $cid => $contextual_element_data) {
    $contextual_element = new stdClass();
    $contextual_element = (object)$contextual_element_data;
    $contextual_element->cid = $cid;
    contextual_elements_save($contextual_element);
  }

  drupal_set_message(t('Changes have been saved.'));
}
